package eu.mhsl.minenet.minigames.instance.game.minigame.types.deathcube;

import eu.mhsl.minenet.minigames.instance.game.minigame.Minigame;
import eu.mhsl.minenet.minigames.util.BatchUtil;
import eu.mhsl.minenet.minigames.instance.Dimension;
import eu.mhsl.minenet.minigames.world.generator.BlockPallet;
import eu.mhsl.minenet.minigames.world.generator.terrain.CircularTerrainGenerator;
import net.minestom.server.coordinate.Pos;
import net.minestom.server.event.player.PlayerMoveEvent;
import net.minestom.server.instance.batch.AbsoluteBlockBatch;
import org.jetbrains.annotations.NotNull;

import java.util.concurrent.CompletableFuture;

class Deathcube extends Minigame {

    final int radius;
    final int height;
    final int percentage;

    public Deathcube(int radius, int height, int percentage) {
        super(Dimension.THE_END.DIMENSION, "Deathcube");
        this.radius = radius;
        this.height = height;
        this.percentage = percentage;
        this.setGenerator(new CircularTerrainGenerator(40, true));
    }

    @Override
    protected void onLoad(CompletableFuture<Void> callback) {
        AbsoluteBlockBatch batch = new AbsoluteBlockBatch();

        for(int x = -radius; x <= radius; x++) {
            for (int z = -radius; z <= radius; z++) {
                if(new Pos(x, 0, z).distance(new Pos(0, 0, 0)) > radius) continue;

                for (int y = 49; y < height; y++) {
                    if(super.rnd.nextInt(1, 100) <= percentage) {
                        batch.setBlock(x, y, z, BlockPallet.WOOD.rnd());
                    }
                }
            }
        }

        BatchUtil.loadAndApplyBatch(batch, this, () -> callback.complete(null));
    }

    @Override
    protected void onPlayerMove(@NotNull PlayerMoveEvent playerMoveEvent) {
        super.onPlayerMove(playerMoveEvent);
        if(isBeforeBeginning) if(playerMoveEvent.getNewPosition().y() > 51.5) playerMoveEvent.setCancelled(true);
        if(playerMoveEvent.getNewPosition().y() > 100) getScore().addResult(playerMoveEvent.getPlayer());
    }

    @Override
    public Pos getSpawn() {
        return new Pos(0, 50, 30);
    }
}
