package eu.mhsl.minenet.minigames.instance.game.minigame.types.stickfight;

import eu.mhsl.minenet.minigames.instance.Dimension;
import eu.mhsl.minenet.minigames.instance.game.minigame.Minigame;
import eu.mhsl.minenet.minigames.util.BatchUtil;
import eu.mhsl.minenet.minigames.world.generator.terrain.CircularTerrainGenerator;
import io.github.bloepiloepi.pvp.config.*;
import io.github.bloepiloepi.pvp.events.FinalAttackEvent;
import net.minestom.server.coordinate.Pos;
import net.minestom.server.entity.Player;
import net.minestom.server.event.player.PlayerMoveEvent;
import net.minestom.server.instance.batch.AbsoluteBlockBatch;
import net.minestom.server.instance.block.Block;
import org.jetbrains.annotations.NotNull;

import java.util.concurrent.CompletableFuture;

public class Stickfight extends Minigame {
    public Stickfight() {
        super(Dimension.OVERWORLD.DIMENSION, "Stickfight");

        eventNode().addChild(
                PvPConfig.emptyBuilder()
                        .damage(DamageConfig.legacyBuilder().fallDamage(false))
                        .attack(AttackConfig.legacyBuilder().attackCooldown(true))
                        .build().createNode()
        );

        eventNode().addListener(FinalAttackEvent.class, finalAttackEvent -> {
            finalAttackEvent.setBaseDamage(0);
            ((Player) finalAttackEvent.getTarget()).setHealth(20);
        });

        setGenerator(new CircularTerrainGenerator(20, false));
    }

    @Override
    protected void onLoad(CompletableFuture<Void> callback) {
        AbsoluteBlockBatch batch = new AbsoluteBlockBatch();
        for (int z = -10; z <= 10; z++) {
            batch.setBlock(0, 50, z, Block.SANDSTONE);
        }
        batch.setBlock(0, 50, 0, Block.GOLD_BLOCK);

        BatchUtil.loadAndApplyBatch(batch, this, () -> callback.complete(null));
    }

    @Override
    protected void onPlayerMove(@NotNull PlayerMoveEvent playerMoveEvent) {
        if(isBeforeBeginning) playerMoveEvent.setCancelled(true);

        if(playerMoveEvent.getNewPosition().y() < 40) {
            playerMoveEvent.getPlayer().teleport(new Pos(0, 51, 0));
        }
    }

    @Override
    public Pos getSpawn() {
        return new Pos(0.5, 51, 0.5);
    }
}
